/*
 * This file is generated by jOOQ.
 */
package com.example.webapp.jooq.enums;

import com.example.webapp.jooq.Public;
import org.jooq.Catalog;
import org.jooq.EnumType;
import org.jooq.Schema;

/** This class is generated by jOOQ. */
@SuppressWarnings({"all", "unchecked", "rawtypes", "this-escape"})
public enum ContractType implements EnumType {
  PACKAGE("PACKAGE"),

  SLAB("SLAB"),

  TRIP("TRIP"),

  ZONE("ZONE");

  private final String literal;

  private ContractType(String literal) {
    this.literal = literal;
  }

  @Override
  public Catalog getCatalog() {
    return getSchema().getCatalog();
  }

  @Override
  public Schema getSchema() {
    return Public.PUBLIC;
  }

  @Override
  public String getName() {
    return "contract_type";
  }

  @Override
  public String getLiteral() {
    return literal;
  }

  /**
   * Lookup a value of this EnumType by its literal. Returns <code>null</code>, if no such value
   * could be found, see {@link EnumType#lookupLiteral(Class, String)}.
   */
  public static ContractType lookupLiteral(String literal) {
    return EnumType.lookupLiteral(ContractType.class, literal);
  }
}
